## GoogleDeploymentManager Queries List
This page contains all queries from GoogleDeploymentManager.

|            Query             |Severity|Category|Description|Help|
|------------------------------|--------|--------|-----------|----|
|Cloud Storage Bucket Is Publicly Accessible<br/><sup><sub>77c1fa3f-83dc-4c9d-bfed-e1d0cc8fd9dc</sub></sup>|<span style="color:#C00">High</span>|Access Control|Cloud Storage Bucket is anonymously or publicly accessible|<a href="https://cloud.google.com/storage/docs/json_api/v1/bucketAccessControls">Documentation</a><br/>|
|SQL DB Instance Backup Disabled<br/><sup><sub>a5bf1a1c-92c7-401c-b4c6-ebdc8b686c01</sub></sup>|<span style="color:#C00">High</span>|Backup|Checks if backup configuration is enabled for all Cloud SQL Database instances|<a href="https://cloud.google.com/sql/docs/mysql/admin-api/rest/v1beta4/instances">Documentation</a><br/>|
|DNSSEC Using RSASHA1<br/><sup><sub>6d7b121a-a2ed-4e37-bd2f-80d9df1dfd35</sub></sup>|<span style="color:#C00">High</span>|Encryption|DNSSEC should not use the RSASHA1 algorithm|<a href="https://cloud.google.com/dns/docs/reference/v1/managedZones">Documentation</a><br/>|
|SQL DB Instance With SSL Disabled<br/><sup><sub>660360d3-9ca7-46d1-b147-3acc4002953f</sub></sup>|<span style="color:#C00">High</span>|Encryption|Cloud SQL Database Instance should have SLL enabled|<a href="https://cloud.google.com/sql/docs/mysql/admin-api/rest/v1beta4/instances">Documentation</a><br/>|
|GKE Legacy Authorization Enabled<br/><sup><sub>df58d46c-783b-43e0-bdd0-d99164f712ee</sub></sup>|<span style="color:#C00">High</span>|Insecure Configurations|Kubernetes Engine Clusters must have Legacy Authorization set to disabled, which means the attribute 'legacyAbac.enabled' must be false.|<a href="https://cloud.google.com/kubernetes-engine/docs/reference/rest/v1/projects.locations.clusters#Cluster.LegacyAbac">Documentation</a><br/>|
|BigQuery Dataset Is Public<br/><sup><sub>83103dff-d57f-42a8-bd81-40abab64c1a7</sub></sup>|<span style="color:#C00">High</span>|Insecure Configurations|BigQuery dataset is anonymously or publicly accessible. Attribute access.specialGroup should not contain 'allAuthenticatedUsers'|<a href="https://cloud.google.com/bigquery/docs/reference/rest/v2/datasets">Documentation</a><br/>|
|COS Node Image Not Used<br/><sup><sub>dbe058d7-b82e-430b-8426-992b2e4677e7</sub></sup>|<span style="color:#C00">High</span>|Insecure Configurations|The node image should be Container-Optimized OS(COS)|<a href="https://cloud.google.com/kubernetes-engine/docs/reference/rest/v1/projects.zones.clusters.nodePools">Documentation</a><br/>|
|IP Aliasing Disabled<br/><sup><sub>28727987-e398-49b8-aef1-8a3e7789d111</sub></sup>|<span style="color:#C00">High</span>|Insecure Configurations|Kubernetes Clusters must be created with Alias IP ranges enabled, which means the attribute 'ipAllocationPolicy' must be defined and the subattribute 'useIpAliases' must be set to 'true'.|<a href="https://cloud.google.com/kubernetes-engine/docs/reference/rest/v1/projects.locations.clusters">Documentation</a><br/>|
|Cluster Master Authentication Disabled<br/><sup><sub>7ef7d141-9fbb-4679-a977-fd0883436906</sub></sup>|<span style="color:#C00">High</span>|Insecure Configurations|Kubernetes Engine Clusters must have Master Authentication set to enabled, which means the attribute 'masterAuth' must have the subattributes 'username' and 'password' defined and not empty|<a href="https://cloud.google.com/kubernetes-engine/docs/reference/rest/v1/projects.locations.clusters">Documentation</a><br/>|
|Private Cluster Disabled<br/><sup><sub>48c61fbd-09c9-46cc-a521-012e0c325412</sub></sup>|<span style="color:#C00">High</span>|Insecure Configurations|Kubernetes Clusters must be created with Private Clusters enabled, meaning the 'privateClusterConfig' must be defined and the attributes 'enablePrivateEndpoint' and 'enablePrivateNodes' must be true.|<a href="https://cloud.google.com/kubernetes-engine/docs/reference/rest/v1/projects.zones.clusters">Documentation</a><br/>|
|MySQL Instance With Local Infile On<br/><sup><sub>c759d6f2-4dd3-4160-82d3-89202ef10d87</sub></sup>|<span style="color:#C00">High</span>|Insecure Configurations|MySQL Instance should not have Local Infile On|<a href="https://cloud.google.com/sql/docs/mysql/admin-api/rest/v1beta4/instances">Documentation</a><br/>|
|Client Certificate Disabled<br/><sup><sub>dd690686-2bf9-4012-a821-f61912dd77be</sub></sup>|<span style="color:#C00">High</span>|Insecure Configurations|Kubernetes Clusters must be created with Client Certificate enabled, which means 'masterAuth' must have 'clientCertificateConfig' with the attribute 'issueClientCertificate' equal to true|<a href="https://cloud.google.com/kubernetes-engine/docs/reference/rest/v1/projects.zones.clusters">Documentation</a><br/>|
|Cluster Labels Disabled<br/><sup><sub>8810968b-4b15-421d-918b-d91eb4bb8d1d</sub></sup>|<span style="color:#C00">High</span>|Insecure Configurations|Kubernetes Clusters must be configured with labels, which means the attribute 'resourceLabels' must be defined|<a href="https://cloud.google.com/kubernetes-engine/docs/reference/rest/v1/projects.zones.clusters">Documentation</a><br/>|
|Network Policy Disabled<br/><sup><sub>c47f90e8-4a19-43f0-8413-cc434d286c4e</sub></sup>|<span style="color:#C00">High</span>|Insecure Configurations|Kubernetes Engine Clusters must have Network Policy enabled, meaning that the attribute 'networkPolicy.enabled' must be true and the attribute 'addonsConfig.networkPolicyConfig.disabled' must be false|<a href="https://cloud.google.com/kubernetes-engine/docs/reference/rest/v1/projects.zones.clusters">Documentation</a><br/>|
|Not Proper Email Account In Use<br/><sup><sub>a21b8df3-c840-4b3d-a41a-10fb2afda171</sub></sup>|<span style="color:#C00">High</span>|Insecure Configurations|Gmail accounts are being used instead of corporate credentials|<a href="https://cloud.google.com/deployment-manager/docs/configuration/set-access-control-resources">Documentation</a><br/>|
|Compute Instance Is Publicly Accessible<br/><sup><sub>8212e2d7-e683-49bc-bf78-d6799075c5a7</sub></sup>|<span style="color:#C00">High</span>|Networking and Firewall|Compute instances shouldn't be accessible from the Internet.|<a href="https://cloud.google.com/compute/docs/reference/rest/v1/instances">Documentation</a><br/>|
|GKE Master Authorized Networks Disabled<br/><sup><sub>62c8cf50-87f0-4295-a974-8184ed78fe02</sub></sup>|<span style="color:#C00">High</span>|Networking and Firewall|Master authorized networks must be enabled in GKE clusters|<a href="https://cloud.google.com/kubernetes-engine/docs/reference/rest/v1/projects.zones.clusters">Documentation</a><br/>|
|Cloud Storage Bucket Versioning Disabled<br/><sup><sub>ad0875c1-0b39-4890-9149-173158ba3bba</sub></sup>|<span style="color:#C00">High</span>|Observability|Cloud Storage Bucket should be enabled|<a href="https://cloud.google.com/storage/docs/json_api/v1/buckets">Documentation</a><br/>|
|Stackdriver Logging Disabled<br/><sup><sub>95601b9a-7fe8-4aee-9b58-d36fd9382dfc</sub></sup>|<span style="color:#C00">High</span>|Observability|Kubernetes Engine Clusters must have Stackdriver Logging enabled, which means the attribute 'loggingService' must be defined and different from 'none'|<a href="https://cloud.google.com/kubernetes-engine/docs/reference/rest/v1/projects.zones.clusters">Documentation</a><br/>|
|Stackdriver Monitoring Disabled<br/><sup><sub>bbfc97ab-e92a-4a7b-954c-e88cec815011</sub></sup>|<span style="color:#C00">High</span>|Observability|Kubernetes Engine Clusters must have Stackdriver Monitoring enabled, which means the attribute 'monitoringService' must be defined and different than 'none'|<a href="https://cloud.google.com/kubernetes-engine/docs/reference/rest/v1/projects.zones.clusters">Documentation</a><br/>|
|Node Auto Upgrade Disabled<br/><sup><sub>dc5c5fee-6c53-43b0-ab11-4c660e064aaf</sub></sup>|<span style="color:#C00">High</span>|Resource Management|Kubernetes nodes must have auto upgrades set to true, which means the attribute 'nodePools' must be defined and the subattribute 'managment' must be defined and have the attribute 'autoUpgrade' set to true|<a href="https://cloud.google.com/kubernetes-engine/docs/reference/rest/v1/projects.zones.clusters">Documentation</a><br/>|
|Disk Encryption Disabled<br/><sup><sub>fc040fb6-4c23-4c0d-b12a-39edac35debb</sub></sup>|<span style="color:#C60">Medium</span>|Encryption|VM disks for critical VMs must be encrypted with Customer Supplied Encryption Keys (CSEK) or with Customer-managed encryption keys (CMEK), which means the attribute 'diskEncryptionKey' must be defined and its sub attributes 'rawKey' or 'kmsKeyName' must also be defined|<a href="https://cloud.google.com/compute/docs/reference/rest/v1/instances">Documentation</a><br/>|
|OSLogin Is Disabled In VM Instance<br/><sup><sub>e66e1b71-c810-4b4e-a737-0ab59e7f5e41</sub></sup>|<span style="color:#C60">Medium</span>|Insecure Configurations|VM instance should have OSLogin enabled|<a href="https://cloud.google.com/compute/docs/reference/rest/v1/instances">Documentation</a><br/>|
|Google Storage Bucket Level Access Disabled<br/><sup><sub>1239f54b-33de-482a-8132-faebe288e6a6</sub></sup>|<span style="color:#C60">Medium</span>|Insecure Configurations|Google Storage Bucket Level Access should be enabled|<a href="https://cloud.google.com/storage/docs/json_api/v1/buckets">Documentation</a><br/>|
|Shielded VM Disabled<br/><sup><sub>9038b526-4c19-4928-bca2-c03d503bdb79</sub></sup>|<span style="color:#C60">Medium</span>|Insecure Configurations|Compute instances must be launched with Shielded VM enabled, which means the attribute 'shieldedInstanceConfig' must be defined and its sub attributes 'enableSecureBoot', 'enableVtpm' and 'enableIntegrityMonitoring' must be set to true|<a href="https://cloud.google.com/compute/docs/reference/rest/v1/instances">Documentation</a><br/>|
|Cloud Storage Anonymous or Publicly Accessible<br/><sup><sub>63ae3638-a38c-4ff4-b616-6e1f72a31a6a</sub></sup>|<span style="color:#C60">Medium</span>|Insecure Configurations|Cloud Storage Buckets must not be anonymously or publicly accessible, which means the subattribute 'entity' from attributes 'acl' and 'defaultObjectAcl' must not be 'allUsers' or 'allAuthenticatedUsers'|<a href="https://cloud.google.com/storage/docs/json_api/v1/buckets">Documentation</a><br/>|
|Cloud DNS Without DNSSEC<br/><sup><sub>313d6deb-3b67-4948-b41d-35b699c2492e</sub></sup>|<span style="color:#C60">Medium</span>|Insecure Configurations|DNSSEC must be enabled for Cloud DNS|<a href="https://cloud.google.com/dns/docs/reference/v1/managedZones">Documentation</a><br/>|
|IP Forwarding Enabled<br/><sup><sub>7c98538a-81c6-444b-bf04-e60bc3ceeec0</sub></sup>|<span style="color:#C60">Medium</span>|Networking and Firewall|Instances must not have IP forwarding enabled, which means the attribute 'canIpForward' must not be true|<a href="https://cloud.google.com/compute/docs/reference/rest/v1/instances">Documentation</a><br/>|
|RDP Access Is Not Restricted<br/><sup><sub>50cb6c3b-c878-4b88-b50e-d1421bada9e8</sub></sup>|<span style="color:#C60">Medium</span>|Networking and Firewall|Check if the Google compute firewall allows unrestricted RDP access. Allowed ports should not contain RDP port 3389|<a href="https://cloud.google.com/compute/docs/reference/rest/v1/firewalls">Documentation</a><br/>|
|SSH Access Is Not Restricted<br/><sup><sub>dee21308-2a7a-49de-8ff7-c9b87e188575</sub></sup>|<span style="color:#C60">Medium</span>|Networking and Firewall|Check if Google Firewall allows SSH access (port 22) from the Internet (public CIDR block)|<a href="https://cloud.google.com/compute/docs/reference/rest/v1/firewalls">Documentation</a><br/>|
|Bucket Without Versioning<br/><sup><sub>227c2f58-70c6-4432-8e9a-a89c1a548cf5</sub></sup>|<span style="color:#C60">Medium</span>|Observability|Bucket should have versioning enabled|<a href="https://cloud.google.com/storage/docs/json_api/v1/buckets">Documentation</a><br/>|
|Project-wide SSH Keys Are Enabled In VM Instances<br/><sup><sub>6e2b1ec1-1eca-4eb7-9d4d-2882680b4811</sub></sup>|<span style="color:#C60">Medium</span>|Secret Management|VM Instance should block project-wide SSH keys|<a href="https://cloud.google.com/compute/docs/reference/rest/v1/instances">Documentation</a><br/>|
